<?php

namespace Codemanas\WooPreviewEmailsPro\Admin;

class AdminArea {
	private static ?AdminArea $_instance = null;
	private string $page_url;
	private string $preview_page;
	private string $license_page;

	public static function get_instance(): ?AdminArea {
		return ( self::$_instance == null ) ? self::$_instance = new self() : self::$_instance;
	}

	private function __construct() {
		$this->page_url     = add_query_arg( [ 'page' => 'codemanas-woocommerce-preview-emails' ], admin_url( 'admin.php' ) );
		$this->preview_page = add_query_arg( [ 'tab' => 'basic' ], $this->page_url );
		$this->license_page = add_query_arg( [ 'tab' => 'license' ], $this->page_url );

		//show tabs - render licensing field
		add_action( 'woo_preview_emails_before_form', array( $this, 'tab_nav' ) );
		add_filter( 'woo_preview_emails_tabs', array( $this, 'maybe_show_licensing_tab' ) );

		//Licensing
		add_action( 'admin_init', array( $this, 'maybe_activate_license' ), 20 );
		add_action( 'admin_init', [ $this, 'licensing_notices' ] );
	}

	public function get_license_page(): string {
		return $this->license_page;
	}

	/**
	 * Renders the tabs for the navigation menu.
	 *
	 * This method generates the HTML markup for the tabs in the navigation menu.
	 * It uses the "tab" query parameter to determine the active tab and applies
	 * the corresponding CSS class to highlight it.
	 *
	 * @return void
	 */
	public function tab_nav(): void {
		$tab        = filter_input( INPUT_GET, 'tab' );
		$active_tab = ! empty( $tab ) ? htmlspecialchars( $tab ) : 'basic';
		?>
        <h2 class="nav-tab-wrapper">
            <a href="<?php echo esc_url( $this->preview_page ); ?>" class="nav-tab
            <?php echo ( 'basic' === $active_tab ) ? esc_attr( 'nav-tab-active' ) : ''; ?>">
				<?php esc_html_e( 'General Settings', 'woo-preview-emails' ); ?>
            </a>
            <a href="<?php echo esc_url( $this->license_page ); ?>" class="nav-tab
            <?php echo ( 'license' === $active_tab ) ? esc_attr( 'nav-tab-active' ) : ''; ?>">
				<?php esc_html_e( 'Licensing', 'woo-preview-emails' ); ?>
            </a>
        </h2>
		<?php
	}

	/**
	 * Checks if the "license" tab should be shown and handles its display.
	 *
	 * @param  bool  $result  The initial value of the result variable.
	 *
	 * @return bool The updated value of the result variable.
	 */
	public function maybe_show_licensing_tab( bool $result ): bool {
		$tab        = filter_input( INPUT_GET, 'tab' );
		$active_tab = ! empty( $tab ) ? htmlspecialchars( $tab ) : false;
		if ( $active_tab === "license" ) {
			$result = true;

			$license     = get_option( '_woo_preview_emails_license_key' );
			$status      = get_option( '_woo_preview_emails_license_status' );
			$nonce_field = wp_nonce_field( 'woo_preview_pro_license', 'woo_preview_pro_license_nonce', true, false );
			load_template( WOO_PREVIEW_EMAILS_PRO_DIR . '/views/license.php', true, [
				'license' => $license,
				'status'  => $status,
				'nonce'   => $nonce_field,
			] );
		}

		return $result;
	}

	/**
	 * Handles the submission of the license activation form.
	 *
	 * @return void
	 */
	public function maybe_activate_license() {
		$licensingHandler = LicensingHandler::get_instance();
		$licensingHandler->handle_license_submission();
	}

	public function licensing_notices() {
		$status = get_option( '_woo_preview_emails_license_status' );


		if ( isset( $_GET['sl_activation'] ) && ! empty( $_GET['sl_msg'] ) ) {
			switch ( $_GET['sl_activation'] ) {
				case 'false':
					$message = urldecode( $_GET['sl_msg'] );
					NoticeHandler::set_message(
						sprintf( '<p>%s : %s</p>', WOO_PREVIEW_EMAILS_PRO_ITEM_NAME, $message ),
						'error',
						true );
					break;
				case 'true':
				default:
					break;
			}
		} elseif ( empty( $status ) || $status === "invalid" ) {
			NoticeHandler::set_message( '<p><strong>Preview Emails for Woocommerce PRO</strong></p>: Invalid License Key', 'error', true );
		}
	}
}