<?php

namespace Codemanas\WooPreviewEmailsPro\Admin;

use Codemanas\WooPreviewEmailsPro\Bootstrap;

final class LicensingHandler {
	private static $_instance = null;

	public static function get_instance(): ?LicensingHandler {
		return ( self::$_instance == null ) ? self::$_instance = new self() : self::$_instance;
	}

	public function handle_license_submission() {

		$nonce = filter_input( INPUT_POST, 'woo_preview_pro_license_nonce' );

		if ( ! wp_verify_nonce( $nonce, 'woo_preview_pro_license' ) ) {
			return;
		}

		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		// listen for our activate button to be clicked
		if ( isset( $_POST['license_activate'] ) ) {
			$this->activate_license();
		}

		// listen for our deactivate button to be clicked
		if ( isset( $_POST['deactivate_the_license'] ) ) {
			$this->deactivate_license();
		}

		// listen for our deactivate button to be clicked
		if ( isset( $_GET['reset_keys'] ) ) {
			$this->reset_keys();
		}
	}

	/**
	 * Reset keys
	 */
	private function reset_keys(): bool {
		$license    = trim( get_option( '_woo_preview_emails_license_key' ) );
		$api_params = array(
			'edd_action' => 'check_license',
			'license'    => $license,
			'item_name'  => urlencode( WOO_PREVIEW_EMAILS_PRO_ITEM_NAME ),
			'url'        => home_url()
		);

		// Call the custom API.
		$response = wp_remote_post( Bootstrap::$store_url, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );
		if ( is_wp_error( $response ) ) {
			return false;
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );
		if ( $license_data->license == 'valid' ) {
			update_option( '_woo_preview_emails_license_status', 'valid' );
		} else {
			update_option( '_woo_preview_emails_license_status', 'invalid' );
		}

		return true;
	}

	/**
	 * Activate license implementation
	 *
	 * @since 1.0.0
	 */
	private function activate_license() {
		// listen for our activate button to be clicked
		if ( isset( $_POST['license_activate'] ) ) {
			// run a quick security check
			if ( ! check_admin_referer( 'woo_preview_emails_licensing_nonce', 'woo_preview_emails_licensing_nonce' ) ) {
				return;
			}

			//Update License Key First
			$license_field = sanitize_text_field( filter_input( INPUT_POST, 'license_key' ) );
			update_option( '_woo_preview_emails_license_key', $license_field );

			// retrieve the license from the database
			$license = trim( get_option( '_woo_preview_emails_license_key' ) );

			// data to send in our API request
			$api_params = array(
				'edd_action' => 'activate_license',
				'license'    => $license,
				'item_id'    => Bootstrap::$item_id,
				'url'        => home_url(),
			);

			// Call the custom API.
			$response = wp_remote_post( Bootstrap::$store_url, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

			// make sure the response came back okay
			if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
				if ( is_wp_error( $response ) ) {
					$message = $response->get_error_message();
				} else {
					$message = __( 'An error occurred, please try again.', 'woo-preview-emails-pro' );
				}
			} else {
				$license_data = json_decode( wp_remote_retrieve_body( $response ) );
				if ( false === $license_data->success ) {
					switch ( $license_data->error ) {
						case 'expired' :
							$message = sprintf( __( 'Your license key expired on %s.', 'woo-preview-emails-pro' ), date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) ) );
							break;
						case 'disabled' :
						case 'revoked' :
							$message = __( 'Your license key has been disabled.', 'woo-preview-emails-pro' );
							break;
						case 'missing' :
							$message = __( 'Invalid license.', 'woo-preview-emails-pro' );
							break;
						case 'invalid' :
						case 'site_inactive' :
							$message = __( 'Your license is not active for this URL.', 'woo-preview-emails-pro' );
							break;
						case 'item_name_mismatch' :
							$message = sprintf( __( 'This appears to be an invalid license key for %s.', 'woo-preview-emails-pro' ), 'woo-preview-emails-pro' );
							break;
						case 'no_activations_left':
							$message = __( 'Your license key has reached its activation limit.', 'woo-preview-emails-pro' );
							break;
						default :
							$message = __( 'An error occurred, please try again.', 'woo-preview-emails-pro' );
							break;
					}
				}
			}


			// Check if anything passed on a message constituting a failure
			if ( ! empty( $message ) ) {
				$base_url = AdminArea::get_instance()->get_license_page();
				$redirect = add_query_arg(
					[
						'sl_activation' => 'false',
						'sl_msg'        => urlencode( $message )
					],
					$base_url );
				wp_redirect( $redirect );
				exit();
			}

			// $license_data->license will be either "valid" or "invalid"
			update_option( '_woo_preview_emails_license_status', $license_data->license );
			wp_redirect( AdminArea::get_instance()->get_license_page() );
			exit();
		}
	}

	/**
	 * Deactivate license implementaiton
	 *
	 * @since 1.0.0
	 */
	private function deactivate_license() {
		if ( isset( $_POST['deactivate_the_license'] ) ) {

			// run a quick security check
			if ( ! check_admin_referer( 'woo_preview_emails_licensing_nonce', 'woo_preview_emails_licensing_nonce' ) ) {
				return;
			}

			// retrieve the license from the database
			$license = trim( get_option( '_woo_preview_emails_license_key' ) );

			// data to send in our API request
			$api_params = array(
				'edd_action' => 'deactivate_license',
				'license'    => $license,
				'item_id'    => Bootstrap::$item_id,
				'item_name'  => WOO_PREVIEW_EMAILS_PRO_ITEM_NAME,
				'url'        => home_url()
			);

			// Call the custom API.
			$response = wp_remote_post( Bootstrap::$store_url, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

			// make sure the response came back okay
			if ( is_wp_error( $response ) || 200 !== wp_remote_retrieve_response_code( $response ) ) {
				if ( is_wp_error( $response ) ) {
					$message = $response->get_error_message();
				} else {
					$message = __( 'An error occurred, please try again.', 'woo-preview-emails-pro' );
				}
				$base_url = AdminArea::get_instance()->get_license_page();
				$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );
				wp_redirect( $redirect );
				exit();
			}

			// decode the license data
			$license_data = json_decode( wp_remote_retrieve_body( $response ) );
			if ( 200 === wp_remote_retrieve_response_code( $response ) && $license_data->success === false && $license_data->license === 'failed' ) {
				$message  = __( 'An error occurred, please try again.', 'woo-preview-emails-pro' );
				$base_url = AdminArea::get_instance()->get_license_page();
				$redirect = add_query_arg( array( 'sl_activation' => 'false', 'message' => urlencode( $message ) ), $base_url );
				wp_redirect( $redirect );
				exit();
			}

			if ( $license_data->license == 'deactivated' ) {
				delete_option( '_woo_preview_emails_license_status' );
			}

			wp_redirect( AdminArea::get_instance()->get_license_page() );
			exit();
		}
	}
}