<?php

namespace Codemanas\WooPreviewEmailsPro;

use Codemanas\WooPreviewEmailsPro\Admin\AdminArea;
use Codemanas\WooPreviewEmailsPro\Admin\NoticeHandler;
use Codemanas\WooPreviewEmailsPro\Helpers\DependencyManager;
use Codemanas\WooPreviewEmailsPro\WooCommerce\Bookings;
use Codemanas\WooPreviewEmailsPro\WooCommerce\Main as WC_Main;
use Codemanas\WooPreviewEmailsPro\WooCommerce\Membership;
use Codemanas\WooPreviewEmailsPro\WooCommerce\Stripe;
use Codemanas\WooPreviewEmailsPro\WooCommerce\Subscriptions;

class Bootstrap {
	private static ?Bootstrap $_instance = null;
	public static int $item_id = 748;
	public static string $store_url = 'https://www.codemanas.com';

	public static function get_instance(): ?Bootstrap {
		return ( self::$_instance == null ) ? self::$_instance = new self() : self::$_instance;
	}

	/**
	 * Class constructor.
	 *
	 * This method initializes the class and registers the necessary actions.
	 *
	 * @return void
	 * @since 1.0.0
	 *
	 */
	private function __construct() {
		require_once WOO_PREVIEW_EMAILS_PRO_DIR . '/vendor/autoload.php';
		add_action( 'plugins_loaded', array( $this, 'load_modules' ) );
	}

	/**
	 * Loads the necessary modules and initializes the plugin.
	 *
	 * This method will load the required modules and initialize the plugin.
	 * It will first check the dependencies required for the plugin to work properly.
	 * If the dependencies are not satisfied, the method will return without loading the modules.
	 * Otherwise, it will proceed to load the necessary modules and initialize the plugin components.
	 * The modules that will be loaded are:
	 * - WC_Main
	 * - AdminArea
	 * - Bookings
	 * - Membership
	 * - Subscriptions
	 * - Stripe
	 *
	 * @return void
	 */
	public function load_modules() {
		NoticeHandler::get_instance();
		add_action( 'init', array( $this, 'load_textdomain' ) );

		if ( ! $this->check_dependencies() ) {
			return;
		}
		add_action( 'admin_init', [ $this, 'updater' ], 0 );
		WC_Main::get_instance();
		AdminArea::get_instance();

		//Addons Load
		Bookings::get_instance();
		//Membership::get_instance();
		Subscriptions::get_instance();
		//Stripe::get_instance();
	}

	/**
	 * Load the text domain for the plugin.
	 *
	 * This function loads the translation files for the plugin to enable internationalization.
	 *
	 * @return void
	 */
	public function load_textdomain() {
		load_plugin_textdomain( 'woo-preview-emails-pro', false, dirname( plugin_basename( WOO_PREVIEW_EMAILS_PRO_FILE ) ) . '/languages/' );
	}

	/**
	 * Checks the dependencies required for the method to work.
	 *
	 * @return bool Returns true if all dependencies are satisfied, false otherwise.
	 */
	public function check_dependencies(): bool {
		$DependencyManager     = DependencyManager::get_instance();
		$is_base_plugin_active = $DependencyManager->is_free_active();
		$is_woocommerce_active = $DependencyManager->is_woocommerce_active();

		if ( ! $is_base_plugin_active ) {
			$message = sprintf( '<p><strong>"Preview Emails for WooCommerce"</strong> is required for <strong>"%s"</strong> to work</p>', WOO_PREVIEW_EMAILS_PRO_ITEM_NAME );
			NoticeHandler::set_message( $message, 'error', false );

			return false;
		}

		if ( ! $is_woocommerce_active ) {
			$message = sprintf( '<p><strong>"WooCommerce"</strong> is required for <strong>"%s"</strong> to work</p>', WOO_PREVIEW_EMAILS_PRO_ITEM_NAME );
			NoticeHandler::set_message( $message, 'error', false );

			return false;
		}

		return true;
	}

	/**
	 * Initializes the plugin updater.
	 *
	 * @return void
	 */
	public function updater() {
		// uncomment this line for testing
//		set_site_transient( 'update_plugins', null );
		$license_key = trim( get_option( '_woo_preview_emails_license_key' ) );
		new Updater( self::$store_url, WOO_PREVIEW_EMAILS_PRO_FILE, array(
			'version' => WOO_PREVIEW_EMAILS_PRO_VERSION,
			'license' => $license_key,
			'author'  => 'codemanas',
			'beta'    => false,
		) );
	}
}

Bootstrap::get_instance();